import { Component, Host, h, Prop, Listen, Element, Watch, Event, Method, } from '@stencil/core';
import { HTMLDecodeString } from '../../global/global';
import { exclusiveHelper } from './helpers/exclusiveHelper';
/**
 * @slot default - used for checkboxes
 */
export class SasCheckboxGroup {
  constructor() {
    this.formatOptions = [];
    this.isMax = false;
    /**
     * name
     */
    this.name = null;
    /**
     * Label
     */
    this.label = '';
    /**
     * Shows a star symbol behind label
     */
    this.required = false;
    /**
     * Selected values
     */
    this.values = [];
    /**
     * Complete selected data
     */
    this.rawData = [];
    /**
     * Options
     */
    this.options = [];
    /**
     * Show select all checkbox
     */
    this.selectAll = false;
    /**
     * Show select all checkbox text, only works when selectAll is true
     */
    this.selectAllText = 'All';
    /**
     * Display checkboxes in x columns
     */
    this.columns = 1;
  }
  watchValuesHandler() {
    this.updateStatus();
  }
  watchOptionsHandler() {
    this.updateStatus();
  }
  componentWillLoad() {
    this.init();
  }
  componentDidLoad() {
    if (this.columns > 1) {
      let container = this.el.shadowRoot.querySelector('.container');
      container.style.gridTemplateColumns = `repeat(${this.columns}, minmax(0, 1fr))`;
    }
  }
  checkboxUpdateHandler(event) {
    this.updateValues(event.detail);
    let thisClone = this;
    clearTimeout(this.timer);
    this.timer = setTimeout(function () {
      thisClone.checkboxGroupUpdate.emit(thisClone.rawData);
      thisClone.checkboxgroupupdate.emit(thisClone.rawData);
    }, 10);
  }
  /**
   * Trigger checkboxgroupupdate event programmatically
   */
  async update() {
    this.checkboxGroupUpdate.emit(this.rawData);
    this.checkboxgroupupdate.emit(this.rawData);
  }
  init() {
    this.convertLightDomToShadowDom();
    this.updateStatus();
  }
  convertLightDomToShadowDom() {
    const checkboxes = this.el.querySelectorAll('sas-checkbox');
    if (checkboxes && checkboxes.length > 0) {
      let lightDomValues = [];
      let lightDomOptions = [];
      checkboxes.forEach((checkbox) => {
        const option = {
          value: checkbox.value,
          text: checkbox.text,
          disabled: checkbox.disabled,
          exclusive: checkbox.exclusive,
        };
        if (checkbox.checked) {
          lightDomValues.push(checkbox.value);
        }
        lightDomOptions.push(option);
      });
      this.el.innerHTML = '';
      if (this.values.length === 0) {
        this.values = lightDomValues;
      }
      this.options = lightDomOptions;
    }
  }
  updateStatus() {
    this.formatOptions = this.getFormatOptions(this.options);
    let validValues = this.getValidValues(this.values, this.formatOptions);
    if (!this.compareArray(this.values, validValues)) {
      this.values = [...validValues];
    }
    this.rawData = this.getRawData(this.values, this.formatOptions);
  }
  getFormatOptions(options) {
    const formatOptions = options.map(option => {
      if ('sub' in option) {
        const formatSubOptions = option.sub.map((subOption) => {
          return {
            value: subOption.value,
            text: HTMLDecodeString(subOption.text),
            disabled: subOption.disabled ? true : false,
            exclusive: subOption.exclusive ? true : false,
          };
        });
        return {
          value: option.value,
          text: HTMLDecodeString(option.text),
          disabled: option.disabled ? true : false,
          sub: formatSubOptions,
        };
      }
      else {
        return {
          value: option.value,
          text: HTMLDecodeString(option.text),
          disabled: option.disabled ? true : false,
          exclusive: option.exclusive ? true : false,
        };
      }
    });
    return formatOptions;
  }
  getValidValues(values, options) {
    const existValues = this.getExistValues(values, options);
    const validValuesByCheckExclusive = this.getValidValuesByCheckExclusive(existValues, options);
    const validValues = this.getValidValuesByCheckMax(validValuesByCheckExclusive, this.max);
    return validValues;
  }
  getExistValues(values, options) {
    let validValues = [];
    if (options.length === 0) {
      validValues = [...values];
    }
    else {
      for (let i = 0; i < values.length; i++) {
        for (let j = 0; j < options.length; j++) {
          if (options[j].sub) {
            for (let m = 0; m < options[j].sub.length; m++) {
              if (options[j].sub[m].value === values[i]) {
                validValues.push(values[i]);
              }
            }
          }
          else {
            if (options[j].value === values[i]) {
              validValues.push(values[i]);
            }
          }
        }
      }
    }
    return validValues;
  }
  getValidValuesByCheckExclusive(values, options) {
    const exclusiveValue = exclusiveHelper.getExclusiveValue(values, options);
    if (exclusiveValue !== null) {
      return [exclusiveValue];
    }
    else {
      return [...values];
    }
  }
  getValidValuesByCheckMax(values, max) {
    if (max && values.length >= max) {
      this.isMax = true;
      return values.slice(0, max);
    }
    else {
      this.isMax = false;
      return [...values];
    }
  }
  compareArray(arrayOne, arrayTwo) {
    return (arrayOne.length === arrayTwo.length
      && arrayOne.every((element, index) => element === arrayTwo[index]));
  }
  getRawData(values, options) {
    let rawData = [];
    if (values.length > 0) {
      values.forEach((value) => {
        options.forEach((option) => {
          if (option.sub && option.sub.length > 0) {
            option.sub.forEach((sub) => {
              if (sub.value === value) {
                const item = rawData.find(item => item.value === option.value);
                if (item) {
                  item.sub.push(sub);
                }
                else {
                  rawData.push({
                    value: option.value,
                    text: option.text,
                    sub: [sub],
                  });
                }
              }
            });
          }
          else {
            if (option.value === value) {
              rawData.push(option);
            }
          }
        });
      });
    }
    return rawData;
  }
  updateValues(checkboxObj) {
    if (checkboxObj.isChecked) {
      const exclusiveValue = exclusiveHelper.getExclusiveValue(this.values, this.options);
      if (checkboxObj.exclusive || exclusiveValue !== null) {
        this.values = [checkboxObj.value];
      }
      else {
        this.values = [...this.values, checkboxObj.value];
      }
    }
    else {
      this.values = this.values.filter(value => value !== checkboxObj.value);
    }
  }
  getCheckboxGroupElements() {
    if (this.selectAll) {
      const selectAllObj = this.getSelectAllObj();
      return h("sas-checkbox-subgroup", { id: "selectAllSubGroup", text: this.selectAllText, icon: selectAllObj.icon, status: selectAllObj.status, checked: selectAllObj.isChecked }, this.getCheckboxElements());
    }
    return this.getCheckboxElements();
  }
  getSelectAllObj() {
    let totalOptions = 0;
    this.formatOptions.forEach((option) => {
      if ('sub' in option) {
        option.sub.forEach(() => {
          totalOptions += 1;
        });
      }
      else {
        totalOptions += 1;
      }
    });
    let selectAllObj = {
      status: 'none',
      isChecked: false,
      icon: '',
    };
    if (this.values.length === 0) {
      selectAllObj.status = 'none';
      selectAllObj.isChecked = false;
      selectAllObj.icon = '';
    }
    else if (this.values.length > 0 && this.values.length < totalOptions) {
      selectAllObj.status = 'part';
      selectAllObj.isChecked = true;
      selectAllObj.icon = 'fas fa-horizontal-rule';
    }
    else {
      selectAllObj.status = 'all';
      selectAllObj.isChecked = true;
      selectAllObj.icon = 'fas fa-check';
    }
    return selectAllObj;
  }
  getCheckboxElements() {
    let elements = [];
    if (this.formatOptions.length > 0) {
      this.formatOptions.forEach((option) => {
        let element;
        if ('sub' in option) {
          let subElements = [];
          let count = 0;
          option.sub.forEach((sub) => {
            const subElement = this.getCheckboxElement(sub);
            if (subElement.$attrs$.checked) {
              count += 1;
            }
            subElements.push(subElement);
          });
          let status;
          let isSubChecked;
          let icon;
          if (count === 0) {
            status = 'none';
            isSubChecked = false;
            icon = '';
          }
          else if (count > 0 && count < option.sub.length) {
            status = 'part';
            isSubChecked = true;
            icon = 'fas fa-horizontal-rule';
          }
          else {
            status = 'all';
            isSubChecked = true;
            icon = 'fas fa-check';
          }
          let isDisabled = false;
          if (this.isMax) {
            isDisabled = true;
          }
          element = h("sas-checkbox-subgroup", { value: option.value, text: option.text, status: status, checked: isSubChecked, disabled: isDisabled, icon: icon }, subElements);
        }
        else {
          element = this.getCheckboxElement(option);
        }
        elements.push(element);
      });
      return elements;
    }
  }
  getCheckboxElement(obj) {
    let isChecked = false;
    if (this.values.length > 0) {
      isChecked = this.values.includes(obj.value);
    }
    let isDisabled = obj.disabled;
    if (this.isMax && !this.values.includes(obj.value)) {
      isDisabled = true;
    }
    const subElement = h("sas-checkbox", { tabindex: -1, value: obj.value, text: obj.text, checked: isChecked, disabled: isDisabled, exclusive: obj.exclusive });
    return subElement;
  }
  render() {
    return (h(Host, { tabindex: -1 },
      h("div", { class: "container" },
        Boolean(this.label)
          ? h("label", { class: "label" },
            this.label,
            this.required
              ? h("span", { class: "required-symbol" }, "*")
              : null)
          : null,
        this.getCheckboxGroupElements(),
        h("slot", null))));
  }
  static get is() { return "sas-checkbox-group"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-checkbox-group.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-checkbox-group.css"]
  }; }
  static get properties() { return {
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "name"
      },
      "attribute": "name",
      "reflect": true,
      "defaultValue": "null"
    },
    "label": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Label"
      },
      "attribute": "label",
      "reflect": true,
      "defaultValue": "''"
    },
    "required": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Shows a star symbol behind label"
      },
      "attribute": "required",
      "reflect": true,
      "defaultValue": "false"
    },
    "values": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "CheckboxValue[]",
        "resolved": "CheckboxValue[]",
        "references": {
          "CheckboxValue": {
            "location": "import",
            "path": "../sas-checkbox/type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Selected values"
      },
      "defaultValue": "[]"
    },
    "rawData": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "any[]",
        "resolved": "any[]",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Complete selected data"
      },
      "defaultValue": "[]"
    },
    "max": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Maximum selected values, not use this prop when has sub group"
      },
      "attribute": "max",
      "reflect": true
    },
    "options": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "Option[]",
        "resolved": "Option[]",
        "references": {
          "Option": {
            "location": "import",
            "path": "./type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Options"
      },
      "defaultValue": "[]"
    },
    "selectAll": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Show select all checkbox"
      },
      "attribute": "select-all",
      "reflect": false,
      "defaultValue": "false"
    },
    "selectAllText": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Show select all checkbox text, only works when selectAll is true"
      },
      "attribute": "select-all-text",
      "reflect": false,
      "defaultValue": "'All'"
    },
    "columns": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Display checkboxes in x columns"
      },
      "attribute": "columns",
      "reflect": true,
      "defaultValue": "1"
    }
  }; }
  static get events() { return [{
      "method": "checkboxGroupUpdate",
      "name": "checkboxGroupUpdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the values are changed by user interaction (will be deprecated)"
      },
      "complexType": {
        "original": "Array<object>",
        "resolved": "object[]",
        "references": {
          "Array": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "checkboxgroupupdate",
      "name": "checkboxgroupupdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the values are changed by user interaction"
      },
      "complexType": {
        "original": "Array<object>",
        "resolved": "object[]",
        "references": {
          "Array": {
            "location": "global"
          }
        }
      }
    }]; }
  static get methods() { return {
    "update": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Trigger checkboxgroupupdate event programmatically",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "values",
      "methodName": "watchValuesHandler"
    }, {
      "propName": "options",
      "methodName": "watchOptionsHandler"
    }]; }
  static get listeners() { return [{
      "name": "checkboxUpdate",
      "method": "checkboxUpdateHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
