import { Component, Host, h, Element, Prop, Method, } from '@stencil/core';
export class SasForm {
  constructor() {
    /**
     * Form submit method
     */
    this.method = 'POST';
    /**
     * Form submit url
     */
    this.url = '';
  }
  /**
   * Clear all fields
   */
  async reset() {
    const formElements = this.el.querySelectorAll('[name]');
    formElements.forEach((element) => {
      let typeElement;
      if (element.tagName === 'SAS-MULTISELECT') {
        typeElement = element;
        typeElement.values = [];
      }
      else if (element.tagName === 'SAS-CHECKBOX' || element.tagName === 'SAS-RADIO' || element.tagName === 'SAS-SWITCH') {
        typeElement = element;
        typeElement.checked = false;
      }
      else if (element.tagName === 'SAS-UPLOAD') {
        typeElement = element;
        typeElement.files = [];
      }
      else if (element.tagName === 'SAS-SELECT') {
        typeElement = element;
        typeElement.value = null;
      }
      else if (element.tagName === 'SAS-DATE-RANGE') {
        typeElement = element;
        typeElement.from = '';
        typeElement.to = '';
      }
      else {
        typeElement = element;
        typeElement.value = '';
      }
      typeElement.resetErrorStatus();
    });
  }
  /**
   * Validate data
   */
  async validate(dataType = '', data = {} || new FormData()) {
    let isValid = true;
    if (dataType === '') {
      const jsonData = await this.getJsonData();
      if ('firstInvalidElementNameUsedInternal' in jsonData) {
        const element = this.el.querySelector(`[name='${jsonData['firstInvalidElementNameUsedInternal']}']`);
        element.scrollIntoView({ behavior: 'smooth' });
        isValid = false;
      }
    }
    else if (dataType === 'json') {
      if ('firstInvalidElementNameUsedInternal' in data) {
        const element = this.el.querySelector(`[name='${data['firstInvalidElementNameUsedInternal']}']`);
        element.scrollIntoView({ behavior: 'smooth' });
        isValid = false;
      }
    }
    else {
      if (data.has('firstInvalidElementNameUsedInternal')) {
        const element = this.el.querySelector(`[name='${data.get('firstInvalidElementNameUsedInternal')}'`);
        element.scrollIntoView({ behavior: 'smooth' });
        isValid = false;
      }
    }
    return isValid;
  }
  /**
   * Submit data
   */
  async submit(method = this.method, url = this.url, additionalData = [], dataType = 'form') {
    let response;
    if (Boolean(method) && Boolean(url)) {
      if (dataType === 'form') {
        let formData = await this.getFormData();
        const isValid = await this.validate('form', formData);
        if (isValid) {
          if (additionalData.length > 0) {
            additionalData.forEach((data) => {
              formData.append(data.key, data.value);
            });
          }
          console.log(formData);
          response = await fetch(url, {
            method: method,
            body: formData
          });
        }
        else {
          response = {
            ok: false,
            status: 400,
            statusText: 'Invalid input',
          };
        }
      }
      else {
        let jsonData = await this.getJsonData();
        const isValid = await this.validate('json', jsonData);
        if (isValid) {
          if (additionalData.length > 0) {
            additionalData.forEach((data) => {
              jsonData[data.key] = data.value;
            });
          }
          response = await fetch(url, {
            method: method,
            headers: {
              'Content-Type': 'application/json',
            },
            body: JSON.stringify(jsonData),
          });
        }
        else {
          response = {
            ok: false,
            status: 400,
            statusText: 'Invalid input',
          };
        }
      }
    }
    else {
      response = {
        ok: false,
        status: 400,
        statusText: 'Bad Request',
      };
    }
    return response;
  }
  async getJsonData() {
    let jsonData = {};
    let firstInvalidElementName = null;
    const formElements = this.el.querySelectorAll('[name]');
    for (let i = 0; i < formElements.length; i++) {
      let typeElement;
      if (formElements[i].tagName === 'SAS-MULTISELECT') {
        typeElement = formElements[i];
        await typeElement.updateErrorStatus();
        if (typeElement.validity.valid) {
          jsonData[typeElement.name] = typeElement.values.join(',');
        }
        else {
          firstInvalidElementName = firstInvalidElementName || typeElement.name;
        }
      }
      else if (formElements[i].tagName === 'SAS-CHECKBOX' || formElements[i].tagName === 'SAS-RADIO' || formElements[i].tagName === 'SAS-SWITCH') {
        typeElement = formElements[i];
        await typeElement.updateErrorStatus();
        if (typeElement.validity.valid) {
          jsonData[typeElement.name] = typeElement.checked;
        }
        else {
          firstInvalidElementName = firstInvalidElementName !== null && firstInvalidElementName !== void 0 ? firstInvalidElementName : typeElement.name;
        }
      }
      else if (formElements[i].tagName === 'SAS-DATE-RANGE') {
        typeElement = formElements[i];
        await typeElement.updateErrorStatus();
        if (typeElement.validity.valid) {
          let data = {
            from: typeElement.from,
            to: typeElement.to,
          };
          jsonData[typeElement.name] = data;
        }
        else {
          firstInvalidElementName = firstInvalidElementName || typeElement.name;
        }
      }
      else if (formElements[i].tagName === 'SAS-DATE-FIELD') {
        typeElement = formElements[i];
        await typeElement.updateErrorStatus();
        if (typeElement.validity.valid) {
          let date = typeElement.value;
          if (date) {
            const localDate = new Date(date);
            date = localDate.toISOString();
          }
          else {
            date = '';
          }
          jsonData[typeElement.name] = date;
        }
        else {
          firstInvalidElementName = firstInvalidElementName !== null && firstInvalidElementName !== void 0 ? firstInvalidElementName : typeElement.name;
        }
      }
      else {
        typeElement = formElements[i];
        await typeElement.updateErrorStatus();
        if (typeElement.validity.valid) {
          let value = typeElement.value;
          if (value && typeElement.tagName === 'SAS-INPUT') {
            value = value.trim();
          }
          jsonData[typeElement.name] = value;
        }
        else {
          firstInvalidElementName = firstInvalidElementName || typeElement.name;
        }
      }
    }
    if (firstInvalidElementName !== null) {
      jsonData = {
        firstInvalidElementNameUsedInternal: firstInvalidElementName,
      };
    }
    return jsonData;
  }
  async getFormData() {
    let formData = new FormData();
    let firstInvalidElementName = null;
    const formElements = this.el.querySelectorAll('[name]');
    for (let i = 0; i < formElements.length; i++) {
      let typeElement;
      if (formElements[i].tagName === 'SAS-MULTISELECT') {
        typeElement = formElements[i];
        await typeElement.updateErrorStatus();
        if (typeElement.validity.valid) {
          formData.append(typeElement.name, typeElement.values.join(','));
        }
        else {
          firstInvalidElementName = firstInvalidElementName || typeElement.name;
        }
      }
      else if (formElements[i].tagName === 'SAS-CHECKBOX' || formElements[i].tagName === 'SAS-RADIO' || formElements[i].tagName === 'SAS-SWITCH') {
        typeElement = formElements[i];
        await typeElement.updateErrorStatus();
        if (typeElement.validity.valid) {
          formData.append(typeElement.name, typeElement.checked);
        }
        else {
          firstInvalidElementName = firstInvalidElementName || typeElement.name;
        }
      }
      else if (formElements[i].tagName === 'SAS-UPLOAD') {
        typeElement = formElements[i];
        await typeElement.updateErrorStatus();
        if (typeElement.validity.valid) {
          const files = typeElement.files;
          files.forEach((file) => {
            if (file instanceof File) {
              formData.append(typeElement.name, file);
            }
          });
          let currentFileNumber = '0';
          if (files.length > 0) {
            currentFileNumber = files.length.toString();
          }
          formData.append('fileNumberWebComponent', currentFileNumber);
          let removedRecordIdString = '';
          if (typeElement.removedFiles.length > 0) {
            const removedRecordIdArray = typeElement.removedFiles.map(file => file.recordId);
            removedRecordIdString = removedRecordIdArray.join(',');
          }
          formData.append('removedRecordIdWebComponent', removedRecordIdString);
        }
        else {
          firstInvalidElementName = firstInvalidElementName || typeElement.name;
        }
      }
      else if (formElements[i].tagName === 'SAS-DATE-RANGE') {
        typeElement = formElements[i];
        await typeElement.updateErrorStatus();
        if (typeElement.validity.valid) {
          let data = {
            from: typeElement.from,
            to: typeElement.to,
          };
          formData.append(typeElement.name, JSON.stringify(data));
        }
        else {
          firstInvalidElementName = firstInvalidElementName || typeElement.name;
        }
      }
      else if (formElements[i].tagName === 'SAS-DATE-FIELD') {
        typeElement = formElements[i];
        await typeElement.updateErrorStatus();
        if (typeElement.validity.valid) {
          let date = typeElement.value;
          if (date) {
            const localDate = new Date(date);
            date = localDate.toISOString();
          }
          else {
            date = '';
          }
          formData.append(typeElement.name, date);
        }
        else {
          firstInvalidElementName = firstInvalidElementName || typeElement.name;
        }
      }
      else {
        typeElement = formElements[i];
        await typeElement.updateErrorStatus();
        if (typeElement.validity.valid) {
          let value = typeElement.value;
          if (typeElement.tagName === 'SAS-INPUT') {
            value = value.trim();
          }
          formData.append(typeElement.name, value);
        }
        else {
          firstInvalidElementName = firstInvalidElementName || typeElement.name;
        }
      }
    }
    if (firstInvalidElementName !== null) {
      formData = new FormData();
      formData.append('firstInvalidElementNameUsedInternal', firstInvalidElementName);
    }
    return formData;
  }
  render() {
    return (h(Host, null,
      h("slot", null)));
  }
  static get is() { return "sas-form"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-form.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-form.css"]
  }; }
  static get properties() { return {
    "method": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Form submit method"
      },
      "attribute": "method",
      "reflect": true,
      "defaultValue": "'POST'"
    },
    "url": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Form submit url"
      },
      "attribute": "url",
      "reflect": true,
      "defaultValue": "''"
    }
  }; }
  static get methods() { return {
    "reset": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          },
          "HTMLSasMultiselectElement": {
            "location": "global"
          },
          "HTMLSasCheckboxElement": {
            "location": "global"
          },
          "HTMLSasRadioElement": {
            "location": "global"
          },
          "HTMLSasSwitchElement": {
            "location": "global"
          },
          "HTMLSasUploadElement": {
            "location": "global"
          },
          "HTMLSasSelectElement": {
            "location": "global"
          },
          "HTMLSasDateRangeElement": {
            "location": "global"
          },
          "HTMLSasInputElement": {
            "location": "global"
          },
          "HTMLSasDateFieldElement": {
            "location": "global"
          },
          "HTMLSasTimeFieldElement": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Clear all fields",
        "tags": []
      }
    },
    "validate": {
      "complexType": {
        "signature": "(dataType?: string, data?: {}) => Promise<boolean>",
        "parameters": [{
            "tags": [],
            "text": ""
          }, {
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "FormData": {
            "location": "global"
          }
        },
        "return": "Promise<boolean>"
      },
      "docs": {
        "text": "Validate data",
        "tags": []
      }
    },
    "submit": {
      "complexType": {
        "signature": "(method?: string, url?: string, additionalData?: any[], dataType?: string) => Promise<any>",
        "parameters": [{
            "tags": [],
            "text": ""
          }, {
            "tags": [],
            "text": ""
          }, {
            "tags": [],
            "text": ""
          }, {
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<any>"
      },
      "docs": {
        "text": "Submit data",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
}
