import { Component, Host, h, Prop, Element, Listen, Method, Watch, Event, } from '@stencil/core';
import { HTMLDecodeString } from '../../global/global';
/**
 * @slot default - used for list item
 */
export class SasList {
  constructor() {
    this.decodeOptions = [];
    /**
     * Enable toggle sub list
     */
    this.toggle = false;
    /**
     * Open current sub list only
     */
    this.compact = false;
    /**
     * Show border
     */
    this.border = false;
    /**
     * If list items have subtext, set list item subtext position
     */
    this.subtextPosition = 'bottom';
    /**
     * Selected value
     */
    this.value = null;
    /**
     * Complete selected data
     */
    this.rawData = null;
    /**
     * Selected text
     */
    this.text = null;
    /**
     * Default value
     */
    this.defaultValue = null;
    /**
     * Options
     */
    this.options = [];
  }
  watchValueHandler() {
    const listItems = this.getListItemElements();
    // not validate when no options and list items, as in this case, options are not loaded
    if (this.decodeOptions.length !== 0 || listItems.length !== 0) {
      const isValid = this.validateValue();
      if (!isValid) {
        this.value = this.defaultValue;
      }
    }
    const hasShadowDom = this.checkShadowDom();
    if (!hasShadowDom) {
      // update select status in light dom
      this.updateSelectStatus();
    }
    this.text = this.getText();
    this.rawData = this.updateRawData();
  }
  watchOptionsHandler() {
    this.decodeOptions = this.getDecodeOptions();
    if (this.decodeOptions.length > 0) {
      const isValid = this.validateValue();
      if (!isValid) {
        this.value = this.defaultValue;
        this.text = this.getText();
      }
    }
    else {
      this.value = null;
      this.text = this.getText();
    }
  }
  componentWillLoad() {
    // if in light dom and has pre select element
    const hasShadowDom = this.checkShadowDom();
    if (!hasShadowDom) {
      let listItems = this.getListItemElements();
      if (listItems.length > 0) {
        listItems.forEach((listItem) => {
          listItem.subtextPosition = this.subtextPosition;
        });
      }
      const selectedElement = this.getSelectedElement();
      if (selectedElement) {
        this.value = selectedElement.value;
        this.text = selectedElement.text;
        this.rawData = this.updateRawData();
      }
    }
    else { // if in shadow dom, validate value
      this.decodeOptions = this.getDecodeOptions();
      if (this.value !== null) {
        const isValid = this.validateValue();
        if (!isValid) {
          this.value = this.defaultValue;
        }
        this.text = this.getText();
        this.rawData = this.updateRawData();
      }
    }
  }
  componentDidRender() {
    if (this.toggle) {
      let subListElements;
      const hasShadowDom = this.checkShadowDom();
      if (hasShadowDom) {
        subListElements = this.el.shadowRoot.querySelectorAll('sas-sublist');
      }
      else {
        subListElements = this.el.querySelectorAll('sas-sublist');
      }
      subListElements.forEach((subListElement) => {
        if (subListElement.toggle !== this.toggle) {
          subListElement.toggle = this.toggle;
        }
        if (subListElement.compact !== this.compact) {
          subListElement.compact = this.compact;
        }
      });
    }
  }
  listItemClickedHandler(event) {
    if (this.value !== event.detail.value) {
      const hasShadowDom = this.checkShadowDom();
      if (!hasShadowDom) {
        // update select status in light dom
        this.updateSelectStatus();
      }
      // update value
      this.value = event.detail.value;
      // update text
      this.text = event.detail.text;
      // update raw data
      this.rawData = this.updateRawData();
      // emit event
      this.listUpdate.emit(event.detail);
    }
  }
  listItemClickEventUsedInternalHandler() {
    const hasShadowDom = this.checkShadowDom();
    let sublists;
    if (hasShadowDom) {
      sublists = this.el.shadowRoot.querySelectorAll('sas-sublist');
    }
    else {
      sublists = this.el.querySelectorAll('sas-sublist');
    }
    sublists.forEach((sublist) => {
      if (!sublist.hideSub && this.compact) {
        sublist.hideSub = true;
      }
      if (sublist.selected) {
        sublist.selected = false;
      }
    });
  }
  /**
   * Trigger inputupdate event programmatically
   */
  async update() {
    const text = this.getText();
    const obj = {
      value: this.value,
      text: text,
    };
    this.listUpdate.emit(obj);
  }
  getDecodeOptions() {
    const decodeOptions = this.options.map(option => {
      if ('sub' in option) {
        const decodeSubOptions = option.sub.map((subOption) => {
          return Object.assign(Object.assign({}, subOption), { text: HTMLDecodeString(subOption.text), subtext: HTMLDecodeString(subOption.subtext) });
        });
        return Object.assign(Object.assign({}, option), { text: HTMLDecodeString(option.text), subtext: HTMLDecodeString(option.subtext), sub: decodeSubOptions });
      }
      else {
        return Object.assign(Object.assign({}, option), { text: HTMLDecodeString(option.text), subtext: HTMLDecodeString(option.subtext) });
      }
    });
    return decodeOptions;
  }
  getText() {
    const hasShadowDom = this.checkShadowDom();
    if (this.value === null) {
      return null;
    }
    else {
      if (hasShadowDom) {
        let result;
        this.decodeOptions.forEach((option) => {
          if ('sub' in option) {
            option.sub.forEach((sub) => {
              if (this.value === sub.value) {
                result = sub;
              }
            });
          }
          else {
            if (this.value === option.value) {
              result = option;
            }
          }
        });
        return result.text;
      }
      else {
        const selectedElement = this.getSelectedElement();
        return selectedElement.text;
      }
    }
  }
  validateValue() {
    const hasShadowDom = this.checkShadowDom();
    let isFound = false;
    if (hasShadowDom) {
      this.decodeOptions.forEach((option) => {
        if ('sub' in option) {
          option.sub.forEach((sub) => {
            if (sub.value === this.value) {
              isFound = true;
            }
          });
        }
        else {
          if (option.value === this.value) {
            isFound = true;
          }
        }
      });
    }
    else {
      const listItems = this.getListItemElements();
      listItems.forEach((listItem) => {
        if (listItem.value === this.value) {
          isFound = true;
        }
      });
    }
    return isFound;
  }
  checkShadowDom() {
    if (this.options.length > 0) {
      return true;
    }
    return false;
  }
  updateSelectStatus() {
    const listItems = this.getListItemElements();
    listItems.forEach((listItem) => {
      if (listItem.value === this.value) {
        listItem.selected = true;
      }
      else {
        listItem.selected = false;
      }
    });
  }
  updateRawData() {
    const value = this.value;
    const text = this.getText();
    let rawData;
    const hasShadowDom = this.checkShadowDom();
    if (hasShadowDom) {
      this.decodeOptions.forEach((option) => {
        if ('sub' in option) {
          option.sub.forEach((sub) => {
            if (this.value === sub.value) {
              rawData = {
                value: option.value,
                text: option.text,
                sub: {
                  value: value,
                  text: text,
                },
              };
            }
          });
        }
        else {
          if (this.value === option.value) {
            rawData = {
              value: value,
              text: text,
            };
          }
        }
      });
    }
    else {
      const selectedElement = this.getSelectedElement();
      if (selectedElement) {
        const parentNode = selectedElement.parentNode;
        if (parentNode.nodeName === 'SAS-SUBLIST') {
          rawData = {
            value: parentNode.value,
            text: parentNode.text,
            sub: {
              value: value,
              text: text,
            },
          };
        }
        else {
          rawData = {
            value: value,
            text: text,
          };
        }
      }
    }
    return rawData;
  }
  getListItemElements() {
    return this.el.querySelectorAll('sas-list-item');
  }
  getSelectedElement() {
    const listItems = this.getListItemElements();
    let selectedElement = null;
    listItems.forEach((listItem) => {
      if (listItem.value === this.value || listItem.selected) {
        selectedElement = listItem;
      }
    });
    return selectedElement;
  }
  generateListItemElements() {
    let elements = [];
    this.decodeOptions.forEach((option) => {
      let element;
      if ('sub' in option) {
        let subElements = [];
        option.sub.forEach((sub) => {
          const subElement = this.getListItemElement(sub);
          subElements.push(subElement);
        });
        element = h("sas-sublist", { value: option.value, text: option.text }, subElements);
      }
      else {
        element = this.getListItemElement(option);
      }
      elements.push(element);
    });
    return elements;
  }
  getListItemElement(obj) {
    let isSelected = false;
    if (this.value !== null && obj.value === this.value) {
      isSelected = true;
    }
    const element = h("sas-list-item", { value: obj.value, text: obj.text, subtext: obj.subtext, disabled: obj.disabled, subtextPosition: this.subtextPosition, selected: isSelected });
    return element;
  }
  render() {
    return (h(Host, null,
      h("div", { class: "container" },
        this.checkShadowDom()
          ? this.generateListItemElements()
          : null,
        h("slot", null))));
  }
  static get is() { return "sas-list"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-list.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-list.css"]
  }; }
  static get properties() { return {
    "toggle": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Enable toggle sub list"
      },
      "attribute": "toggle",
      "reflect": true,
      "defaultValue": "false"
    },
    "compact": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Open current sub list only"
      },
      "attribute": "compact",
      "reflect": true,
      "defaultValue": "false"
    },
    "border": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Show border"
      },
      "attribute": "border",
      "reflect": true,
      "defaultValue": "false"
    },
    "subtextPosition": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'bottom' | 'right'",
        "resolved": "\"bottom\" | \"right\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If list items have subtext, set list item subtext position"
      },
      "attribute": "subtext-position",
      "reflect": true,
      "defaultValue": "'bottom'"
    },
    "value": {
      "type": "any",
      "mutable": true,
      "complexType": {
        "original": "ListItemValue",
        "resolved": "boolean | number | string",
        "references": {
          "ListItemValue": {
            "location": "import",
            "path": "../sas-list-item/type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Selected value"
      },
      "attribute": "value",
      "reflect": true,
      "defaultValue": "null"
    },
    "rawData": {
      "type": "any",
      "mutable": true,
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Complete selected data"
      },
      "attribute": "raw-data",
      "reflect": false,
      "defaultValue": "null"
    },
    "text": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Selected text"
      },
      "attribute": "text",
      "reflect": true,
      "defaultValue": "null"
    },
    "defaultValue": {
      "type": "any",
      "mutable": false,
      "complexType": {
        "original": "ListItemValue",
        "resolved": "boolean | number | string",
        "references": {
          "ListItemValue": {
            "location": "import",
            "path": "../sas-list-item/type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Default value"
      },
      "attribute": "default-value",
      "reflect": true,
      "defaultValue": "null"
    },
    "options": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "Option[]",
        "resolved": "Option[]",
        "references": {
          "Option": {
            "location": "import",
            "path": "./type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Options"
      },
      "defaultValue": "[]"
    }
  }; }
  static get events() { return [{
      "method": "listUpdate",
      "name": "listUpdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the right icon is clicked by user"
      },
      "complexType": {
        "original": "object",
        "resolved": "object",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "update": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Trigger inputupdate event programmatically",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "value",
      "methodName": "watchValueHandler"
    }, {
      "propName": "options",
      "methodName": "watchOptionsHandler"
    }]; }
  static get listeners() { return [{
      "name": "listItemClicked",
      "method": "listItemClickedHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "listItemClickEventUsedInternal",
      "method": "listItemClickEventUsedInternalHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
