import { Component, Host, h, Prop, Listen, Element, Watch, Event, Method, } from '@stencil/core';
/**
 * @slot default - used for radios
 */
export class SasRadioGroup {
  constructor() {
    /**
     * name
     */
    this.name = null;
    /**
     * Label
     */
    this.label = '';
    /**
     * Shows a star symbol behind label
     */
    this.required = false;
    /**
     * Checked value
     */
    this.value = null;
    /**
     * Default value
     */
    this.defaultValue = null;
    /**
     * Checked text
     */
    this.text = null;
    /**
     * Complete checked data
     */
    this.rawData = null;
    /**
     * Options
     */
    this.options = [];
  }
  watchValueHandler() {
    const radioElements = this.getRadioElements();
    // not validate when no options and list items, as in this case, options are not loaded
    if (this.options.length !== 0 || radioElements.length !== 0) {
      const isValid = this.validateValue();
      if (!isValid) {
        this.value = this.defaultValue;
      }
    }
    const hasShadowDom = this.checkShadowDom();
    if (!hasShadowDom) {
      // update check status in light dom
      this.updateCheckStatus();
    }
    this.text = this.getText();
    this.rawData = this.updateRawData();
  }
  watchOptionsHandler() {
    if (this.options.length > 0) {
      const isValid = this.validateValue();
      if (!isValid) {
        this.value = this.defaultValue;
        this.text = this.getText();
      }
    }
    else {
      this.value = null;
      this.text = this.getText();
    }
  }
  componentWillLoad() {
    // if in light dom and has pre select element
    const hasShadowDom = this.checkShadowDom();
    if (!hasShadowDom) {
      const checkedElement = this.getCheckedElement();
      if (checkedElement) {
        this.value = checkedElement.value;
        this.text = checkedElement.text;
        this.rawData = this.updateRawData();
      }
    }
    else { // if in shadow dom, validate value
      if (this.value !== null) {
        const isValid = this.validateValue();
        if (!isValid) {
          this.value = this.defaultValue;
          this.text = this.getText();
          this.rawData = this.updateRawData();
        }
      }
    }
  }
  radioUpdateHandler(event) {
    if (this.value !== event.detail.value) {
      const hasShadowDom = this.checkShadowDom();
      if (!hasShadowDom) {
        // update check status in light dom
        this.updateCheckStatus();
      }
      // update value
      this.value = event.detail.value;
      // update text
      this.text = event.detail.text;
      // update raw data
      this.rawData = this.updateRawData();
      // emit event
      this.radioGroupUpdate.emit(event.detail);
      this.radiogroupupdate.emit(event.detail);
    }
  }
  /**
   * Trigger radiogroupupdate event programmatically
   */
  async update() {
    const text = this.getText();
    const obj = {
      value: this.value,
      text: text,
    };
    this.radioGroupUpdate.emit(obj);
    this.radiogroupupdate.emit(obj);
  }
  getText() {
    const hasShadowDom = this.checkShadowDom();
    if (this.value === null) {
      return null;
    }
    else {
      if (hasShadowDom) {
        let result;
        this.options.forEach((option) => {
          if ('sub' in option) {
            option.sub.forEach((sub) => {
              if (this.value === sub.value) {
                result = sub;
              }
            });
          }
          else {
            if (this.value === option.value) {
              result = option;
            }
          }
        });
        return result.text;
      }
      else {
        const checkedElement = this.getCheckedElement();
        return checkedElement.text;
      }
    }
  }
  validateValue() {
    const hasShadowDom = this.checkShadowDom();
    let isFound = false;
    if (hasShadowDom) {
      this.options.forEach((option) => {
        if ('sub' in option) {
          option.sub.forEach((sub) => {
            if (sub.value === this.value) {
              isFound = true;
            }
          });
        }
        else {
          if (option.value === this.value) {
            isFound = true;
          }
        }
      });
    }
    else {
      const radioElements = this.getRadioElements();
      radioElements.forEach((radioElement) => {
        if (radioElement.value === this.value) {
          isFound = true;
        }
      });
    }
    return isFound;
  }
  checkShadowDom() {
    if (this.options.length > 0) {
      return true;
    }
    return false;
  }
  updateCheckStatus() {
    const radioElements = this.getRadioElements();
    radioElements.forEach((radioElement) => {
      if (radioElement.value === this.value) {
        radioElement.checked = true;
      }
      else {
        radioElement.checked = false;
      }
    });
  }
  updateRawData() {
    const value = this.value;
    const text = this.getText();
    let rawData;
    const hasShadowDom = this.checkShadowDom();
    if (hasShadowDom) {
      this.options.forEach((option) => {
        if ('sub' in option) {
          option.sub.forEach((sub) => {
            if (this.value === sub.value) {
              rawData = {
                value: option.value,
                text: option.text,
                sub: {
                  value: value,
                  text: text,
                },
              };
            }
          });
        }
        else {
          if (this.value === option.value) {
            rawData = {
              value: value,
              text: text,
            };
          }
        }
      });
    }
    else {
      const checkedElement = this.getCheckedElement();
      if (checkedElement) {
        rawData = {
          value: value,
          text: text,
        };
      }
    }
    return rawData;
  }
  getRadioElements() {
    return this.el.querySelectorAll('sas-radio');
  }
  getCheckedElement() {
    const radioElements = this.getRadioElements();
    let checkedElement = null;
    radioElements.forEach((radioElement) => {
      if (radioElement.value === this.value || radioElement.checked) {
        checkedElement = radioElement;
      }
    });
    return checkedElement;
  }
  getRadioGroupElements() {
    let elements = [];
    this.options.forEach((option) => {
      let isChecked = false;
      if (this.value !== null && option.value === this.value) {
        isChecked = true;
      }
      let element = h("sas-radio", { value: option.value, text: option.text, checked: isChecked, disabled: option.disabled });
      elements.push(element);
    });
    return elements;
  }
  render() {
    return (h(Host, null,
      h("div", { class: "container" },
        Boolean(this.label)
          ? h("label", { class: "label" },
            this.label,
            this.required
              ? h("span", { class: "required-symbol" }, "*")
              : null)
          : null,
        this.getRadioGroupElements(),
        h("slot", null))));
  }
  static get is() { return "sas-radio-group"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-radio-group.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-radio-group.css"]
  }; }
  static get properties() { return {
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "name"
      },
      "attribute": "name",
      "reflect": true,
      "defaultValue": "null"
    },
    "label": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Label"
      },
      "attribute": "label",
      "reflect": true,
      "defaultValue": "''"
    },
    "required": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Shows a star symbol behind label"
      },
      "attribute": "required",
      "reflect": true,
      "defaultValue": "false"
    },
    "value": {
      "type": "any",
      "mutable": true,
      "complexType": {
        "original": "RadioValue",
        "resolved": "boolean | number | string",
        "references": {
          "RadioValue": {
            "location": "import",
            "path": "../sas-radio/type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Checked value"
      },
      "attribute": "value",
      "reflect": false,
      "defaultValue": "null"
    },
    "defaultValue": {
      "type": "any",
      "mutable": false,
      "complexType": {
        "original": "RadioValue",
        "resolved": "boolean | number | string",
        "references": {
          "RadioValue": {
            "location": "import",
            "path": "../sas-radio/type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Default value"
      },
      "attribute": "default-value",
      "reflect": true,
      "defaultValue": "null"
    },
    "text": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Checked text"
      },
      "attribute": "text",
      "reflect": true,
      "defaultValue": "null"
    },
    "rawData": {
      "type": "any",
      "mutable": true,
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Complete checked data"
      },
      "attribute": "raw-data",
      "reflect": false,
      "defaultValue": "null"
    },
    "options": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "Option[]",
        "resolved": "Option[]",
        "references": {
          "Option": {
            "location": "import",
            "path": "./type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Options"
      },
      "defaultValue": "[]"
    }
  }; }
  static get events() { return [{
      "method": "radioGroupUpdate",
      "name": "radioGroupUpdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the value is changed by user interaction (will be deprecated)"
      },
      "complexType": {
        "original": "object",
        "resolved": "object",
        "references": {}
      }
    }, {
      "method": "radiogroupupdate",
      "name": "radiogroupupdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the value is changed by user interaction"
      },
      "complexType": {
        "original": "object",
        "resolved": "object",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "update": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Trigger radiogroupupdate event programmatically",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "value",
      "methodName": "watchValueHandler"
    }, {
      "propName": "options",
      "methodName": "watchOptionsHandler"
    }]; }
  static get listeners() { return [{
      "name": "radioupdate",
      "method": "radioUpdateHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
