import { Component, Host, h, Element, Prop, Watch, Method, } from '@stencil/core';
import { keydownEvent, HTMLDecodeString } from '../../global/global';
export class SasUpload {
  constructor() {
    this.filterMessage = '';
    this.needReplace = false;
    this.MIME_TYPE = {
      pdf: 'application/pdf',
      docx: 'application/msword',
      doc: 'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
      jpg: 'image/jpeg',
      jpeg: 'image/jpeg',
      png: 'image/png',
      gif: 'image/gif',
      xls: 'application/vnd.ms-excel',
      xlsx: 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
      mp4: 'video/mp4',
      mov: 'video/quicktime',
      wmv: 'video/x-ms-asf, video/x-ms-wmv',
      avi: 'application/x-troff-msvideo, video/avi, video/msvideo, video/x-msvideo'
    };
    this.acceptTypes = [];
    /**
     * Compact
     */
    this.compact = false;
    /**
     * Input outside the file container, used when input and file container are separate
     */
    this.inputOutside = false;
    /**
     * Show full image
     */
    this.fullImage = false;
    /**
     * Type
     */
    this.type = 'default';
    /**
     * Label
     */
    this.label = '';
    /**
     * name
     */
    this.name = null;
    /**
     * Max
     */
    this.max = null;
    /**
     * Maximum file size
     */
    this.size = 30;
    /**
     * Accept file type, eg: ".jpg,.jpeg", set this value will overwrite current accept file type
     */
    this.accept = '';
    /**
     * Array of file object
     */
    this.files = [];
    /**
     * Removed files
     */
    this.removedFiles = [];
    /**
     * Submit method
     */
    this.method = 'POST';
    /**
     * Submit url
     */
    this.url = '';
    /**
     * Required
     */
    this.required = false;
    /**
     * Disabled
     */
    this.disabled = false;
    /**
     * Error
     */
    this.error = false;
    /**
     * Validity
     */
    this.validity = {
      valid: true,
      valueMissing: false,
    };
    /**
     * Only shows when prop error is true, auto generated
     */
    this.message = '';
    this.triggerInputClick = () => {
      const input = this.el.shadowRoot.querySelector('#input');
      input.click();
    };
    this.handleDeleteIconClick = (event) => {
      const removedFile = this.files[event.target.index];
      if (!(removedFile instanceof File)) {
        this.removedFiles = this.removedFiles.concat(removedFile);
      }
      this.files = this.files.filter((_file, index) => index !== event.target.index);
    };
    this.handleInputChange = (event) => {
      const filterFiles = this.filter(event.target.files);
      if (filterFiles.length > 0) {
        if (this.needReplace) {
          this.files.forEach((file) => {
            if (!(file instanceof File)) {
              this.removedFiles = this.removedFiles.concat(file);
            }
          });
          this.files = filterFiles;
        }
        else {
          this.files = [...this.files, ...filterFiles];
        }
      }
      this.resetInput();
    };
    this.handleDrop = (event) => {
      event.preventDefault();
      if (!this.disabled) {
        const filterFiles = this.filter(event.dataTransfer.files);
        if (filterFiles.length > 0) {
          if (this.needReplace) {
            this.files = filterFiles;
          }
          else {
            this.files = [...this.files, ...filterFiles];
          }
        }
      }
      this.removeDropEffect();
    };
    this.resetInput = () => {
      // remove attached file
      const input = this.el.shadowRoot.querySelector('#input');
      input.value = '';
    };
    this.addDropEffect = (event) => {
      event.preventDefault();
      if (!this.disabled) {
        const container = this.el.shadowRoot.querySelector('.container');
        container.classList.add('drop-effect');
      }
    };
    this.removeDropEffect = () => {
      if (!this.disabled) {
        const container = this.el.shadowRoot.querySelector('.container');
        container.classList.remove('drop-effect');
      }
    };
    this.removeAllFiles = () => {
      const removedFile = this.files[0];
      if (!(removedFile instanceof File)) {
        this.removedFiles = this.removedFiles.concat(removedFile);
      }
      this.files = [];
    };
  }
  watchHandler() {
    this.resetErrorStatus();
  }
  componentWillLoad() {
    if (this.accept !== '') {
      this.acceptTypes = this.accept.replace(/ /g, '').replace(/\./g, '').split(',');
    }
    else {
      if (this.type === 'image') {
        this.acceptTypes = ['jpg', 'jpeg', 'png', 'gif'];
      }
      else if (this.type === 'doc') {
        this.acceptTypes = ['xls', 'xlsx', 'pdf', 'docx', 'doc'];
      }
      else {
        this.acceptTypes = ['xls', 'xlsx', 'pdf', 'docx', 'doc', 'jpg', 'jpeg', 'png', 'gif'];
      }
    }
  }
  /**
   * Validate component value. If invalid, scroll component into view.
   */
  async validate() {
    await this.updateErrorStatus();
    if (!this.validity.valid) {
      this.el.scrollIntoView({ behavior: 'smooth' });
    }
  }
  /**
   * Submit file
   */
  async submit(method = this.method, url = this.url, additionalData = [], options = {}) {
    let response;
    let isUploadAll = false;
    if ('uploadAll' in options) {
      isUploadAll = options.uploadAll;
    }
    const isUpdated = await this.checkFileUpdate();
    if (isUpdated || isUploadAll) { // has update || need upload all
      if (Boolean(method) && Boolean(url)) {
        let formData = await this.getFormData(isUploadAll);
        if (additionalData.length > 0) {
          additionalData.forEach((data) => {
            formData.append(data.key, data.value);
          });
        }
        response = await fetch(url, {
          method: method,
          body: formData
        });
      }
      else {
        response = {
          ok: false,
          status: 400,
          statusText: 'Bad Request',
        };
      }
    }
    else { // no update
      response = {
        ok: true,
        status: 200,
        statusText: 'OK',
      };
    }
    return response;
  }
  /**
   * Show error status if value is not valid (only for components internal use)
   */
  async updateErrorStatus() {
    this.validity = this.getValidity();
    this.error = !this.validity.valid;
    if (!this.validity.valid) {
      this.message = this.getErrorMessage();
    }
  }
  /**
   * Clear error status (only for components internal use)
   */
  async resetErrorStatus() {
    this.error = false;
    this.message = '';
  }
  /**
   * Get submit data in FormData type
   */
  async getFormData(uploadAll = false) {
    let existFiles = [];
    let formData = new FormData();
    this.files.forEach((file) => {
      if (file instanceof File) {
        formData.append(file.name, file);
      }
      else {
        existFiles.push(file);
      }
    });
    let currentFileNumber = '0';
    if (this.files.length > 0) {
      currentFileNumber = this.files.length.toString();
    }
    formData.append('fileNumberWebComponent', currentFileNumber);
    if (uploadAll) {
      formData.append('existFiles', JSON.stringify(existFiles));
    }
    else {
      let removedRecordIdString = '';
      if (this.removedFiles.length > 0) {
        let recordIdArray = [];
        this.removedFiles.forEach((file) => {
          if ('recordId' in file) {
            recordIdArray.push(file.recordId);
          }
        });
        removedRecordIdString = recordIdArray.join(',');
      }
      formData.append('removedRecordIdWebComponent', removedRecordIdString);
    }
    return formData;
  }
  /**
   * Check if adding new file or removing old file
   */
  async checkFileUpdate() {
    let isUpdated = false;
    if (this.removedFiles.length > 0) {
      isUpdated = true;
    }
    this.files.forEach((file) => {
      if (file instanceof File) {
        isUpdated = true;
      }
    });
    return isUpdated;
  }
  getErrorMessage() {
    const validity = this.validity;
    if (validity.valueMissing) {
      return `Please attach files`;
    }
  }
  getValidity() {
    const length = this.files.length;
    let valueMissing = false;
    if (this.required && length === 0) {
      valueMissing = true;
    }
    return {
      valid: !valueMissing,
      valueMissing: valueMissing,
    };
  }
  filter(files) {
    this.filterMessage = '';
    const container = this.el.shadowRoot.querySelector('.container');
    container.dataset.filterMessage = this.filterMessage;
    let filteredFiles = [];
    filteredFiles = this.filterMax(files);
    if (!Boolean(this.filterMessage)) {
      filteredFiles = this.filterType(filteredFiles);
    }
    if (!Boolean(this.filterMessage)) {
      filteredFiles = this.filterSize(filteredFiles);
    }
    if (!Boolean(this.filterMessage)) {
      filteredFiles = this.filterDuplicate(filteredFiles);
    }
    container.dataset.filterMessage = this.filterMessage;
    return filteredFiles;
  }
  filterMax(files) {
    const fileTypeMap = {
      default: 'files',
      doc: 'files',
      image: 'images',
    };
    if (this.max === 1) { // single file
      if (files.length === 1) {
        this.needReplace = true;
        return files;
      }
      else {
        this.filterMessage = `Max. ${this.max} ${fileTypeMap[this.type]}`;
        return [];
      }
    }
    else { // multiple files
      if (!Boolean(this.max) || this.files.length + files.length <= this.max) {
        return files;
      }
      else {
        this.filterMessage = `Max. ${this.max} ${fileTypeMap[this.type]}`;
        return [];
      }
    }
  }
  filterType(files) {
    let validFiles = [];
    let invalidFileNames = [];
    for (let i = 0; i < files.length; i++) {
      const type = this.getFileType(files[i].name).toLowerCase();
      const result = this.acceptTypes.findIndex(acceptType => acceptType === type);
      if (result !== -1) {
        validFiles.push(files[i]);
      }
      else {
        invalidFileNames.push(files[i].name);
      }
    }
    if (invalidFileNames.length > 0) {
      this.filterMessage = 'Invalid file type: ' + invalidFileNames.join(', ');
    }
    return validFiles;
  }
  filterSize(files) {
    const maxFileSize = this.size * 1024 * 1024;
    let validFiles = [];
    let invalidFileNames = [];
    for (let i = 0; i < files.length; i++) {
      if (files[i].size <= maxFileSize) {
        validFiles.push(files[i]);
      }
      else {
        invalidFileNames.push(files[i].name);
      }
    }
    if (invalidFileNames.length > 0) {
      this.filterMessage = 'Invalid file size: ' + invalidFileNames.join(', ');
    }
    return validFiles;
  }
  filterDuplicate(files) {
    const originalFiles = this.files;
    if (originalFiles.length === 0) {
      return files;
    }
    else {
      let validFiles = [];
      let invalidFileNames = [];
      const originalFileNames = originalFiles.map(file => file.name);
      for (let i = 0; i < files.length; i++) {
        const result = originalFileNames.findIndex(originalFileName => originalFileName === files[i].name);
        if (result === -1) {
          validFiles.push(files[i]);
        }
        else {
          invalidFileNames.push(files[i].name);
        }
      }
      if (invalidFileNames.length > 0) {
        this.filterMessage = 'Duplicate file: ' + invalidFileNames.join(', ');
      }
      return validFiles;
    }
  }
  getAcceptAttributeValue() {
    let value = '';
    for (let i = 0; i < this.acceptTypes.length; i++) {
      value += (this.MIME_TYPE[this.acceptTypes[i]] + ',');
    }
    return value.slice(0, -1);
  }
  getFiles() {
    const decodeFiles = this.files.map((file) => {
      let fileClone = Object.assign(file);
      if ('url' in fileClone) {
        fileClone.url = HTMLDecodeString(fileClone.url);
      }
      return fileClone;
    });
    if (this.fullImage) {
      return (h("sas-image", { imageObject: decodeFiles[0] },
        h("sas-icon", { class: "full-image-icon", type: "round", icon: "fas fa-times", onClick: this.removeAllFiles })));
    }
    else {
      const fileList = decodeFiles.map((file, index) => {
        return (h("div", { class: "file", key: 'url' in file ? file.url : file.name },
          this.getFileType(file.name) === 'jpg' || this.getFileType(file.name) === 'jpeg' || this.getFileType(file.name) === 'png'
            ? h("sas-image", { class: "thumbnail", imageObject: file })
            : h("sas-icon", { class: "file-type-icon", icon: this.getFileIcon(file.name) }),
          h("div", { class: "file-name-container" },
            h("a", { class: "file-name", href: 'url' in file ? file.url : null, download: 'url' in file ? file.name : false }, file.name)),
          'size' in file
            ? h("span", null, this.getFileSize(file.size))
            : null,
          ('preventDelete' in file && file.preventDelete) || this.disabled
            ? null
            : h("sas-icon", { class: "file-upload-delete-icon", type: "round", icon: "fas fa-times", index: index, onClick: this.handleDeleteIconClick })));
      });
      return fileList;
    }
  }
  getFileIcon(fileName) {
    const fileType = this.getFileType(fileName);
    if (fileType === 'pdf') {
      return 'fas fa-file-pdf';
    }
    if (fileType === 'doc' || fileType === 'docx') {
      return 'fas fa-file-word';
    }
    return 'fas fa-file';
  }
  getFileType(fileName) {
    const fileNameArray = fileName.split('.');
    return fileNameArray[fileNameArray.length - 1].toLowerCase();
  }
  getFileSize(fileSize) {
    const units = ['b', 'kb', 'mb', 'gb'];
    let size = fileSize;
    let index = 0;
    while (size > 1024) {
      size /= 1024;
      index += 1;
    }
    return `${parseFloat(size.toFixed(2))}${units[index]}`;
  }
  render() {
    return (h(Host, null,
      !this.disabled && this.inputOutside
        ? h("span", { tabindex: 0, onClick: this.triggerInputClick, onKeyDown: keydownEvent },
          h("slot", null, "Choose file"),
          h("input", { id: "input", type: "file", accept: this.getAcceptAttributeValue(), multiple: this.max !== 1, onChange: this.handleInputChange }))
        : null,
      this.disabled && this.files.length === 0
        ? null
        : h("div", { class: `container ${this.inputOutside && this.files.length === 0 ? 'container-hide' : ''}`, "data-filter-message": this.filterMessage, "data-error-message": this.message, onDrop: this.handleDrop, onDragOver: this.addDropEffect, onDragEnter: this.addDropEffect, onDragLeave: this.removeDropEffect, onDragEnd: this.removeDropEffect },
          Boolean(this.label)
            ? h("label", { class: "label" },
              this.label,
              this.required
                ? h("span", { class: "required-symbol" }, "*")
                : null)
            : null,
          !this.disabled && !this.inputOutside
            ? h("div", { class: `info-container ${this.files.length > 0 && this.fullImage ? 'hide' : ''}` },
              h("sas-icon", { class: "upload-icon", icon: this.type === 'image'
                  ? "fal fa-camera-alt"
                  : "fal fa-cloud-upload" }),
              h("div", { class: "drop-zone-text" },
                "Drag and drop your ",
                this.type === 'image' ? 'photo' : 'file',
                " here, or",
                h("label", { class: "input-label", tabindex: 0, htmlFor: "input", onKeyDown: keydownEvent }, " browse")),
              h("input", { id: "input", type: "file", accept: this.getAcceptAttributeValue(), multiple: this.max !== 1, onChange: this.handleInputChange }),
              h("div", { class: "tip" },
                "(Accepted files ",
                `.${this.acceptTypes.join(', .')}`,
                " Max size ",
                this.size,
                "mb each)"))
            : null,
          this.files.length > 0
            ? h("div", { class: "file-container" }, this.getFiles())
            : null)));
  }
  static get is() { return "sas-upload"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["sas-upload.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["sas-upload.css"]
  }; }
  static get properties() { return {
    "compact": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Compact"
      },
      "attribute": "compact",
      "reflect": true,
      "defaultValue": "false"
    },
    "inputOutside": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Input outside the file container, used when input and file container are separate"
      },
      "attribute": "input-outside",
      "reflect": true,
      "defaultValue": "false"
    },
    "fullImage": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Show full image"
      },
      "attribute": "full-image",
      "reflect": true,
      "defaultValue": "false"
    },
    "type": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'default' | 'doc' | 'image'",
        "resolved": "\"default\" | \"doc\" | \"image\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Type"
      },
      "attribute": "type",
      "reflect": true,
      "defaultValue": "'default'"
    },
    "label": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Label"
      },
      "attribute": "label",
      "reflect": true,
      "defaultValue": "''"
    },
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "name"
      },
      "attribute": "name",
      "reflect": true,
      "defaultValue": "null"
    },
    "max": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Max"
      },
      "attribute": "max",
      "reflect": true,
      "defaultValue": "null"
    },
    "size": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Maximum file size"
      },
      "attribute": "size",
      "reflect": true,
      "defaultValue": "30"
    },
    "accept": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Accept file type, eg: \".jpg,.jpeg\", set this value will overwrite current accept file type"
      },
      "attribute": "accept",
      "reflect": true,
      "defaultValue": "''"
    },
    "files": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "UploadFile[]",
        "resolved": "UploadFile[]",
        "references": {
          "UploadFile": {
            "location": "import",
            "path": "./type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Array of file object"
      },
      "defaultValue": "[]"
    },
    "removedFiles": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "UploadFile[]",
        "resolved": "UploadFile[]",
        "references": {
          "UploadFile": {
            "location": "import",
            "path": "./type"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Removed files"
      },
      "defaultValue": "[]"
    },
    "method": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Submit method"
      },
      "attribute": "method",
      "reflect": true,
      "defaultValue": "'POST'"
    },
    "url": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Submit url"
      },
      "attribute": "url",
      "reflect": true,
      "defaultValue": "''"
    },
    "required": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Required"
      },
      "attribute": "required",
      "reflect": true,
      "defaultValue": "false"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Disabled"
      },
      "attribute": "disabled",
      "reflect": true,
      "defaultValue": "false"
    },
    "error": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Error"
      },
      "attribute": "error",
      "reflect": true,
      "defaultValue": "false"
    },
    "validity": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "{ valid: boolean; valueMissing: boolean; }",
        "resolved": "{ valid: boolean; valueMissing: boolean; }",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Validity"
      },
      "defaultValue": "{\n    valid: true,\n    valueMissing: false,\n  }"
    },
    "message": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Only shows when prop error is true, auto generated"
      },
      "attribute": "message",
      "reflect": true,
      "defaultValue": "''"
    }
  }; }
  static get methods() { return {
    "validate": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Validate component value. If invalid, scroll component into view.",
        "tags": []
      }
    },
    "submit": {
      "complexType": {
        "signature": "(method?: string, url?: string, additionalData?: any[], options?: Option) => Promise<any>",
        "parameters": [{
            "tags": [],
            "text": ""
          }, {
            "tags": [],
            "text": ""
          }, {
            "tags": [],
            "text": ""
          }, {
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "Option": {
            "location": "import",
            "path": "./type"
          }
        },
        "return": "Promise<any>"
      },
      "docs": {
        "text": "Submit file",
        "tags": []
      }
    },
    "updateErrorStatus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Show error status if value is not valid (only for components internal use)",
        "tags": []
      }
    },
    "resetErrorStatus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Clear error status (only for components internal use)",
        "tags": []
      }
    },
    "getFormData": {
      "complexType": {
        "signature": "(uploadAll?: boolean) => Promise<FormData>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "FormData": {
            "location": "global"
          }
        },
        "return": "Promise<FormData>"
      },
      "docs": {
        "text": "Get submit data in FormData type",
        "tags": []
      }
    },
    "checkFileUpdate": {
      "complexType": {
        "signature": "() => Promise<boolean>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<boolean>"
      },
      "docs": {
        "text": "Check if adding new file or removing old file",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "files",
      "methodName": "watchHandler"
    }]; }
}
